import random
import httpx
import asyncio
import ssl
from httpx import ConnectError, ConnectTimeout, ProxyError, ReadTimeout, ReadError, UnsupportedProtocol
from curl_cffi import requests
from scan.response import Response
from scan.common import logger


class SSLFactory:
    ORIGIN_CIPHERS =['ECDHE+AESGCM', 'ECDHE+CHACHA20', 'DHE+AESGCM', 'DHE+CHACHA20', 'ECDH+AESGCM', 'DH+AESGCM',
                     'ECDH+AES', 'DH+AES', 'RSA+AESGCM', 'RSA+AES']

    def __init__(self):
        self.ciphers = self.ORIGIN_CIPHERS

    def __call__(self) -> ssl.SSLContext:
        random.shuffle(self.ciphers)
        ciphers = ":".join(self.ciphers)
        ciphers = ciphers + ":!aNULL:!eNULL:!MD5:!DSS"
        context = ssl.create_default_context()
        context.set_ciphers(ciphers)
        return context


class Downloader:
    def __init__(self):
        self.ssl_context = None
        self.client_dict = {}
        self.ssl_gen = SSLFactory()

    async def _session_client(self, verify, http2, proxies):
        if isinstance(proxies, dict):
            proxy = proxies.get('https://')
            if not proxy:
                proxy = proxies.get('http://')
            proxy = str(proxy)
        else:
            proxy = str(proxies)
        if proxy in self.client_dict:
            return self.client_dict[proxy]
        else:
            async with asyncio.Lock():
                client = httpx.AsyncClient(proxies=proxies, verify=verify, http2=http2,
                                           event_hooks={'response': [self.log_response]})
                self.client_dict[proxy] = client
                logger.debug(f'新建请求客户端,当前有{len(self.client_dict)}个客户端')
                return client

    @staticmethod
    async def generate_headers():
        return {
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36 Edg/129.0.0.0',
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'sec-fetch-site': 'none',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-user': '?1',
            'sec-fetch-dest': 'document',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'en-US,en;q=0.9',
        }

    async def close_session_client(self, proxies):
        if isinstance(proxies, dict):
            proxy = proxies.get('https://')
            if not proxy:
                proxy = proxies.get('http://')
            proxy = str(proxy)
        else:
            proxy = str(proxies)
        if proxy in self.client_dict:
            client = self.client_dict[proxy]
            await client.aclose()
            del self.client_dict[proxy]

    async def close_all(self):
        try:
            for proxy, client in self.client_dict.items():
                await client.aclose()
            self.client_dict.clear()
        except:
            pass

    @staticmethod
    async def log_request(request):
        pass

    @staticmethod
    async def log_response(response):
        """
        日志钩子
        """
        request = response.request
        if response.status_code not in [200, 301, 302]:
            logger.warning(f'{response.status_code}  {request.url}')
        else:
            logger.debug(f'{response.status_code}  {request.url}')

    async def tls_request(self, method, url, data=None, json=None, proxies=None):
        response = Response()
        try:
            if proxies:
                proxy = {'https': proxies.get('https://'), 'http': proxies.get('http://')}
            else:
                proxy = None
            resp = requests.request(method=method, url=url, data=data, json=json, impersonate="chrome101",
                                    proxies=proxy)
            await self.log_response(resp)
            response.response = resp
            response.ok = True
        except Exception as e:
            logger.error(f'tls request error: {e}')
        return response

    async def request(self, url, params=None, headers=None, cookies=None, auth=None, proxies=None, allow_redirects=True,
                      verify=True, http2=False, content=None, data=None, files=None, json=None, stream=False,
                      timeout=30, cycle=3, tls=False, session=True):
        if verify is True:
            verify = self.ssl_gen()
        if data or json or content:
            method = 'POST'
        else:
            method = 'GET'
        if not headers:
            headers = await self.generate_headers()
        if tls:
            return await self.tls_request(method=method, url=url, data=data, json=json, proxies=proxies)
        response = Response()
        response.request_url = url
        for _ in range(cycle):
            try:
                if session:
                    client = await self._session_client(verify, http2, proxies)
                    resp = await client.request(method=method, url=url, content=content, data=data, files=files,
                                                json=json, params=params, headers=headers, cookies=cookies,
                                                auth=auth, follow_redirects=allow_redirects, timeout=timeout)
                    response.response = resp
                    response.ok = True
                elif stream:
                    client = httpx.AsyncClient(proxies=proxies, event_hooks={'response': [self.log_response]},
                                               verify=verify, http2=http2, follow_redirects=allow_redirects)
                    request = client.build_request(method=method, url=url, headers=headers, cookies=cookies,
                                                   content=content, data=data, files=files, json=json, timeout=timeout)
                    resp = await client.send(request, stream=True)
                    response.response = resp
                    response.ok = True
                    response.client = client  # need aclose
                else:
                    async with httpx.AsyncClient(proxies=proxies, event_hooks={'response': [self.log_response]},
                                                 verify=verify, http2=http2) as client:
                        resp = await client.request(
                            method=method, url=url, content=content, data=data, files=files, json=json, params=params,
                            headers=headers, cookies=cookies, auth=auth, follow_redirects=allow_redirects,
                            timeout=timeout
                        )
                        response.response = resp
                        response.ok = True
                # if resp.status_code == 403:
                #     return await self.tls_request(method=method, url=url, data=data, json=json, proxies=proxies)
                return response
            except ConnectError as e:
                response.message = 'ConnectError'
                logger.error(f'Failed to request {url}  ConnectError:{e}')
            except ConnectTimeout as e:
                response.message = 'ConnectTimeout'
                logger.error(f'Failed to request {url}  ConnectTimeout:{e}')
            except ProxyError as e:
                response.message = 'ProxyError'
                logger.error(f'Failed to request {url}  ProxyError:{e}')
            except ReadTimeout as e:
                response.message = 'ReadTimeout'
                logger.error(f'Failed to request {url}  ReadTimeout:{e}')
            except ReadError as e:
                response.message = 'ReadError'
                logger.error(f'Failed to request {url}  ReadError:{e}')
            except UnsupportedProtocol as e:
                response.message = 'UnsupportedProtocol'
                logger.error(f'Failed to request {url}  UnsupportedProtocol:{e}')
            except Exception as e:
                response.message = e
                logger.error(f'Failed to request {url}  {e}')
        return response



import asyncio
import os
from hashlib import md5
import datetime
from pymongo.errors import DuplicateKeyError
from abc import ABC, abstractmethod
from scan import logger
from scan.config import Config
from scan.core.scheduler import Scheduler
from scan.util import get_local_ip


class Spider(ABC):

    def __init__(self, spider_name=None, cfg_path=None):
        self.spider_status = 'pending'
        if not spider_name:
            self.spider_name = self.__class__.__name__
        else:
            self.spider_name = spider_name
        self.config = Config(cfg_path)
        self.admin = self.config.admin()
        self.rabbitmq = None
        self.mongo_db = None
        self.redis = None
        self.redis_conn = None
        self.kafka = None
        self.oss_conn = None
        self.postgresql = None
        self.task_num = None
        self.spider_id = self._spider_id()
        self.host_ip = None
        self.host_name = None
        self.task_success_count = 0
        self.task_fail_count = 0
        self.scheduler = Scheduler(self)

    def _spider_id(self):
        # Docker
        host_name = os.getenv('CONTAINER_NAME')  # docker run -e CONTAINER_NAME=***
        if not host_name:
            host_name = os.getenv('HOSTNAME')
        # Unix
        if not host_name:
            host_name = os.uname().nodename
        # Windows
        if not host_name:
            host_name = os.environ['COMPUTERNAME']
        self.host_name = host_name
        host_ip = os.getenv('HOST_IP')  # Docker
        if not host_ip:
            host_ip = get_local_ip()
        self.host_ip = host_ip
        spider_id = self.spider_name + '-' + self.host_ip + '-' + self.host_name
        spider_id = md5(spider_id.encode()).hexdigest()[:10]
        return spider_id

    @staticmethod
    def counter(ct=None):
        def handle(func):
            async def wrapper(self, *args, **kwargs):
                result = await func(self, *args, **kwargs)
                if not self.mongo_db:
                    logger.error('There is no connection of mongoDB')
                else:
                    try:
                        collection_name = None
                        if not result and ct is False:
                            collection_name = 'hscan_fail_log'
                        elif result and ct is True:
                            collection_name = 'hscan_success_log'
                        elif ct is None and not result:
                            collection_name = 'hscan_fail_log'
                        elif ct is None and result:
                            collection_name = 'hscan_success_log'
                        if collection_name:
                            today = datetime.date.today()
                            start_of_day = datetime.datetime(today.year, today.month, today.day, 0, 0, 0)
                            timestamp = int(start_of_day.timestamp())
                            _id = f'{self.spider_name}{timestamp}'
                            save_data = {'_id': _id, 'date': str(today), 'timestamp': timestamp,
                                         'name': self.spider_name, 'count': 1}
                            try:
                                db_data = await self.mongo_db.get_collection(collection_name).find_one({'_id': _id})
                                if db_data:
                                    await self.mongo_db.get_collection(collection_name).update_one(
                                        {'_id': _id}, {'$inc': {'count': 1}})
                                else:
                                    await self.mongo_db.get_collection(collection_name).insert_one(save_data)
                            except DuplicateKeyError:
                                await self.mongo_db.get_collection(collection_name).update_one(
                                    {'_id': _id}, {'$inc': {'count': 1}})
                    except Exception as e:
                        logger.error(f'counter process error:{e}')
                if result:
                    return result
                return True
            return wrapper
        return handle

    def get_count(self):
        result = self.task_success_count, self.task_fail_count
        self.task_success_count = self.task_fail_count = 0
        return result

    async def _process(self, task_info):
        """
        在_task中调用
        包一层统计数据
        """
        try:
            res = await self.process(task_info)
            if res:
                self.task_success_count += 1
            else:
                self.task_fail_count += 1
            return res
        except Exception as e:
            self.task_fail_count += 1
            raise e

    @abstractmethod
    async def process(self, task_info):
        """
        爬虫业务逻辑入口
        :param task_info: 队列任务信息
        :return:
        """

    async def stop(self):
        self.spider_status = 'stopped'
        await self.scheduler.stop()

    async def site(self):
        """
        1. 可以对配置文件进行修改
        eg：
            self.config.config.set('client', 'task_num', 1)
        :return:
        2. 可以设置新的成员变量
        eg:
            self.mq2 = ''
        """

    @abstractmethod
    async def init(self):
        """
        初始化数据处理连接
        eg:
            mq_config = self.config.rabbitmq()
            self.rabbitmq = RabbitMQ(host=mq_config.get('host'), port=mq_config.get('port'), user=mq_config.get('user'),
                           password=mq_config.get('password'))
            await self.rabbitmq.init()
        :return:
        """

    @abstractmethod
    async def _task(self):
        """
        :return:
        """

    async def run(self, task_num=1, monitor=False):
        """
        :param task_num: 爬虫并发数
        :param monitor:  是否心跳开启监控
        :return:
        """
        logger.info('爬虫程序初始化开始...')
        # 获取爬虫配置
        await self.site()
        # 初始化数据库配置
        await self.init()
        await self.scheduler.start(monitor)
        self.spider_status = 'running'
        logger.info('爬虫程序初始化结束...')
        logger.info('开始执行任务...')
        task_list = []
        for _ in range(task_num):
            t = asyncio.create_task(self._task())
            task_list.append(t)
        await asyncio.gather(*task_list)
        logger.info('结束执行任务...')
        await self.scheduler.stop()




import asyncio
from abc import ABCMeta
from scan.core.spiders.spider import Spider
from scan import logger


class SimpleSpider(Spider, metaclass=ABCMeta):

    task_queue = asyncio.Queue()

    async def add_url(self, url: str):
        """
        把下次需要请求的url放入队列
        """
        await self.task_queue.put({'url': url})

    async def add_task(self, task_info: dict):
        """
        添加任务对象
        """
        await self.task_queue.put(task_info)

    async def init(self):
        """
        普通抓取不要求初始化配置
        """

    async def _task(self):
        while self.spider_status == 'running' and not self.task_queue.empty():
            try:
                if self.task_queue.empty():
                    await asyncio.sleep(0.1)
                    continue
                try:
                    task_info = self.task_queue.get_nowait()
                except asyncio.QueueEmpty:
                    continue
                try:
                    res = await self._process(task_info)
                finally:
                    self.task_queue.task_done()  # 减少队列计数
            except Exception as e:
                logger.error(e)
        await self.task_queue.join()  # 等待所有任务完成




import io
import asyncio
from contextlib import AsyncExitStack
from aiobotocore.session import AioSession
from scan.common import logger


class AioCFR2:
    def __init__(self, **kwargs):
        self.account_id = kwargs.get('account_id')
        self.aws_access_key_id = kwargs.get('aws_access_key_id')
        self.aws_secret_access_key = kwargs.get('aws_secret_access_key')
        self.session = AioSession()
        self.client = None
        self.lock = asyncio.Lock()

    async def create_s3_client(self):
        exit_stack = AsyncExitStack()
        client = await exit_stack.enter_async_context(
            self.session.create_client('s3',
                                       aws_access_key_id=self.aws_access_key_id,
                                       aws_secret_access_key=self.aws_secret_access_key,
                                       endpoint_url=f'https://{self.account_id}.r2.cloudflarestorage.com')
        )
        return client

    async def initialize(self):
        async with self.lock:
            if not self.client:
                self.client = await self.create_s3_client()

    async def close(self):
        if self.client:
            await self.client.close()

    async def upload(self, bucket_name, resp, file_name):
        """
        :param bucket_name:
        :param resp:
        :param file_name:
        :return:
        小文件上传
        """
        try:
            if not self.client:
                await self.initialize()
            content = resp.content()
            if not content:
                return False
            byte_stream_file_obj = io.BytesIO(content)
            try:
                # 上传字节流文件对象到S3存储桶中
                res = await self.client.put_object(Body=byte_stream_file_obj, Bucket=bucket_name, Key=file_name)
                await resp.aclose()
                if not res:
                    logger.error(f'Uploaded {file_name} to {bucket_name} fail')
                    return False
                if res.get('ResponseMetadata', {}).get('HTTPStatusCode') != 200:
                    logger.error(f'Uploaded {file_name} to {bucket_name} fail')
                    return False
                logger.info(f'Uploaded {file_name} to {bucket_name}')
                return True
            except Exception as e:
                logger.error(f'Error uploading {file_name}: {e}')
            return False
        except Exception as e:
            logger.error(f'Upload process error: {e}')
            await resp.aclose()

    async def upload_bytes(self, bucket_name, file_bytes, file_name):
        """
        :param bucket_name:
        :param file_bytes:
        :param file_name:
        :return:
        小文件上传
        """
        try:
            if not self.client:
                await self.initialize()
            byte_stream_file_obj = io.BytesIO(file_bytes)
            # 上传字节流文件对象到S3存储桶中
            res = await self.client.put_object(Body=byte_stream_file_obj, Bucket=bucket_name, Key=file_name)
            if not res:
                logger.error(f'Uploaded {file_name} to {bucket_name} fail')
                return False
            if res.get('ResponseMetadata', {}).get('HTTPStatusCode') != 200:
                logger.error(f'Uploaded {file_name} to {bucket_name} fail')
                return False
            logger.info(f'Uploaded {file_name} to {bucket_name}')
            return True
        except Exception as e:
            logger.error(f'Error uploading {file_name}: {e}')
        return False

    async def upload_large(self, bucket_name, resp, file_name):
        """
        :param bucket_name:  桶名
        :param resp: 流式请求的响应
        :param file_name: 保存的文件名
        :return: 上传成功或失败
        用于处理大文件上传，不过小文件也可以上传
        """
        try:
            if not self.client:
                await self.initialize()
            # 逐块上传流数据到S3存储桶中
            upload_id = (await self.client.create_multipart_upload(Bucket=bucket_name, Key=file_name)).get('UploadId')
            part_number = 1
            uploaded_parts = []
            part_size = 5 * 1024 * 1024  # 每块5MB，小了要报错

            async for chunk in resp.response.aiter_bytes(chunk_size=part_size):
                if chunk:
                    part = await self.client.upload_part(
                        Body=chunk,
                        Bucket=bucket_name,
                        Key=file_name,
                        PartNumber=part_number,
                        UploadId=upload_id
                    )
                    uploaded_parts.append({'PartNumber': part_number, 'ETag': part['ETag']})
                    part_number += 1

            res = await self.client.complete_multipart_upload(
                Bucket=bucket_name,
                Key=file_name,
                UploadId=upload_id,
                MultipartUpload={'Parts': uploaded_parts}
            )
            await resp.aclose()
            if not res:
                logger.error(f'Uploaded {file_name} to {bucket_name} fail')
                return False
            if res.get('ResponseMetadata', {}).get('HTTPStatusCode') != 200:
                logger.error(f'Uploaded {file_name} to {bucket_name} fail')
                return False
            logger.info(f'Uploaded {file_name} from stream to {bucket_name}')
            return True
        except Exception as e:
            logger.error(f'Error uploading {file_name}: {e}')
            await resp.aclose()
        return False



import datetime
import hashlib


def get_local_ip():
    import socket
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    except:
        pass
    finally:
        s.close()
    return ip


def get_local_name():
    import socket
    return socket.gethostname()


def encrypt(char, method='md5'):
    """
    支持md5和sha1加密方式
    :param char:
    :param method:
    :return:
    """
    char = str(char)
    if method == 'md5':
        m = hashlib.md5()
    elif method == 'sha1':
        m = hashlib.sha1()
    m.update(char.encode('utf8'))
    return m.hexdigest()


def date_to_char(type='s'):
    """
    当前时间转成年月日时分秒形式
    :return:
    """
    if type == 's':
        return datetime.datetime.now().strftime('%Y%m%d%H%M%S')
    elif type == 'm':
        return datetime.datetime.now().strftime('%Y%m%d%H%M')
    elif type == 'd':
        return datetime.datetime.now().strftime('%Y%m%d')


def js_str_to_json(input_str):
    import execjs
    import json
    js_code = """
    function convertToJson(inputString) {
        var jsonObject = eval('(' + inputString + ')');
        var jsonString = JSON.stringify(jsonObject);
        return jsonString;
    }
    """
    ctx = execjs.compile(js_code)
    try:
        output_json_string = ctx.call("convertToJson", input_str)
        data = json.loads(output_json_string)
        return data
    except Exception:
        return



import configparser
from scan import logger


class Config:
    def __init__(self, cfg_file_path):
        if cfg_file_path:
            self.config: configparser.ConfigParser = self.base_config(cfg_file_path)
        else:
            logger.warning('没有指定配置文件')

    @staticmethod
    def base_config(cfg_file_path=None):
        config = configparser.ConfigParser()
        config.read(cfg_file_path, encoding="utf-8")
        return config

    def admin(self):
        return dict(self.config.items('admin'))

    def postgres(self):
        return dict(self.config.items('postgres'))

    def mongo(self):
        return dict(self.config.items('mongo'))

    def redis(self):
        return dict(self.config.items('redis'))

    def rabbitmq(self):
        return dict(self.config.items('rabbitmq'))

    def client(self):
        return dict(self.config.items('client'))

    def oss(self):
        return dict(self.config.items('oss'))

    def kafka(self):
        return dict(self.config.items('kafka'))

    def other(self):
        return dict(self.config.items('other'))

    # @staticmethod
    # def explain(func_text):
    #     """
    #     :param func_text: 函数的字符串形式，函数名必须为parse_func
    #     :return:
    #     """
    #     exec(func_text, globals())
    #     parse_func = eval('parse_func')
    #     return parse_func



from motor.motor_asyncio import AsyncIOMotorCollection
from functools import wraps
from pymongo.errors import DuplicateKeyError


class MongoMiddleware:
    def __init__(self):
        self.insert_success_count = 0
        self.insert_repeat_count = 0
        self.insert_fail_count = 0
        self.update_success_count = 0
        self.update_fail_count = 0
        self._patch_methods()

    def _patch_methods(self):
        original_insert_one = AsyncIOMotorCollection.insert_one
        original_update_one = AsyncIOMotorCollection.update_one

        @wraps(original_insert_one)
        async def patched_insert_one(collection, document, *args, **kwargs):
            """
            插入补丁
            """
            try:
                result = await original_insert_one(collection, document, *args, **kwargs)
                self.insert_success_count += 1
                return result
            except DuplicateKeyError:
                self.insert_repeat_count += 1
                raise
            except Exception:
                self.insert_fail_count += 1
                raise

        @wraps(original_update_one)
        async def patched_update_one(filter, update, *args, **kwargs):
            """
            更新补丁
            """
            try:
                result = await original_update_one(filter, update, *args, **kwargs)
                self.update_success_count += 1
                return result
            except Exception:
                self.update_fail_count += 1
                raise

        AsyncIOMotorCollection.insert_one = patched_insert_one
        AsyncIOMotorCollection.update_one = patched_update_one

    def get_count(self):
        result = self.insert_success_count, self.insert_repeat_count, self.insert_fail_count, \
                 self.update_success_count, self.update_fail_count
        self.insert_success_count = self.insert_repeat_count = self.insert_fail_count = \
            self.update_success_count = self.update_fail_count = 0
        return result



import sys


def get_logger(name=None, log_to_file=False, log_file_path="logfile.log", default_level="DEBUG"):
    from loguru import logger

    # 移除所有现有的处理器
    logger.remove()

    # 添加输出到控制台的处理器，默认级别为 default_level
    logger.add(sys.stdout, colorize=True, level=default_level, format="<green>{time}</green> <level>{message}</level>")
    return logger





import oss2


class OSS:
    def __init__(self, **kwargs):
        self.oss_type = kwargs.get('type') or 'oss'
        self.acc_id = kwargs.get('acc_id')
        self.acc_sec = kwargs.get('acc_sec')
        self.region = kwargs.get('region')
        self.bucket = kwargs.get('bucket')
        self.oss_conn = self.connection()

    def connection(self):
        auth = oss2.Auth(self.acc_id, self.acc_sec)
        oss_con = oss2.Bucket(auth, self.region, self.bucket)
        return oss_con


__all__ = OSS




import asyncio
import aiohttp
from aiohttp import ClientTimeout

from scan.response import Response
from scan.common import logger


class Downloader:
    def __init__(self):
        self.session = None

    @staticmethod
    async def gen_headers():
        return {
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/129.0.0.0 Safari/537.36 Edg/129.0.0.0',
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
            'sec-fetch-site': 'none',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-user': '?1',
            'sec-fetch-dest': 'document',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'en-US,en;q=0.9',
        }

    @staticmethod
    async def log_response(response):
        request = response.request_info
        if response.status not in [200, 301, 302]:
            logger.warning(f'{response.status}  {request.url}')
        else:
            logger.debug(f'{response.status}  {request.url}')

    async def request(self, url, params=None, headers=None, cookies=None, auth=None, proxies=None, allow_redirects=True,
                      verify=True, http2=False, content=None, data=None, files=None, json=None, stream=False,
                      timeout=30, cycle=3, tls=False, session=True):
        if not self.session:
            async with asyncio.Lock():
                self.session = aiohttp.ClientSession(timeout=ClientTimeout(timeout))
        if data or json or content:
            method = 'POST'
        else:
            method = 'GET'
        if not headers:
            headers = await self.gen_headers()
        if proxies and isinstance(proxies, dict):
            proxies = list(proxies.values())[0]
        response = Response()
        response.request_url = url
        for _ in range(cycle):
            try:
                resp = await self.session.request(method=method, url=url, data=data, json=json, params=params,
                                                  headers=headers, cookies=cookies, proxy=proxies, auth=auth,
                                                  allow_redirects=allow_redirects)
                await self.log_response(resp)
                content = await resp.read()
                resp.__setattr__('status_code', resp.status)
                resp.__setattr__('content', content)
                response.response = resp
                return response
            except Exception as e:
                response.message = e
                logger.error(f'Failed to request {url}  {e}')
        return response



import asyncio
import random
from abc import ABCMeta
from scan.core.spiders.spider import Spider
from scan import logger


class RabbitSpider(Spider, metaclass=ABCMeta):

    async def _task(self):
        """
        分布式队列任务，在任务分发使用rabbitmq时
        """
        await asyncio.sleep(random.randint(2, 20))
        task_queue = self.config.rabbitmq().get('task_queue')
        if not self.rabbitmq:
            logger.error('The task queue connection is not initialized')
            return
        arguments = None
        # 优先级队列
        if self.config.rabbitmq().get('priority'):
            arguments = {'x-max-priority': int(self.config.rabbitmq().get('priority'))}
        if self.config.rabbitmq().get('auto_ack'):
            auto_ack = True
        else:
            auto_ack = False
        while self.spider_status == 'running':
            try:
                logger.info('rabbitmq task consume started...')
                await self.rabbitmq.consume(self._process, task_queue, auto_ack=auto_ack, arguments=arguments)
            except Exception as e:
                logger.error(e)



import time
import asyncio
from requests import session
from scan import logger
from threading import Thread

from scan.middleware.mongo_middleware import MongoMiddleware


class Scheduler:
    def __init__(self, spider):
        self.spider = spider
        self.__monitor = False
        self.__last_report_time = 0
        self.__monitor_complete = False
        self._mongo_middleware = MongoMiddleware()
        self.session = session()
        self.admin_url = self.spider.admin.get('url')
        self.admin_token = self.spider.admin.get('token')

    def _heartbeat(self, status):
        """
        发送心跳  0:注册  1：运行中  -1：停止
        :param status:
        :return:
        """
        timestamp = int(time.time())
        try:
            logger.debug(f'发送心跳: {self.spider.spider_id} {status} {timestamp}')
            resp = self.session.post(f'{self.admin_url}/heartbeat', headers={'Authorization': self.admin_token},
                                     json={'id': self.spider.spider_id,
                                           'name': self.spider.spider_name,
                                           'host_ip': self.spider.host_ip,
                                           'host_name': self.spider.host_name,
                                           'status': status,
                                           'timestamp': timestamp},
                                     timeout=30)
            if resp.status_code != 200:
                logger.error(f'发送心跳异常:{resp.status_code} {resp.text}')
        except Exception as e:
            logger.error(f'发送心跳异常:{e}')
        time.sleep(10)
        return timestamp

    def _report(self, immediate=False):
        """
        发送成功失败计数
        1分钟一次
        """
        if immediate or int(time.time()) - self.__last_report_time >= 60:
            self.__last_report_time = int(time.time())
        else:
            return
        timestamp = int(time.time())
        task_success_count, task_fail_count = self.spider.get_count()
        insert_success_count, insert_repeat_count, insert_fail_count, update_success_count, update_fail_count = \
            self._mongo_middleware.get_count()
        try:
            resp = self.session.post(f'{self.admin_url}/report', headers={'Authorization': self.admin_token},
                                     json={'id': self.spider.spider_id,
                                           'name': self.spider.spider_name,
                                           'host_ip': self.spider.host_ip,
                                           'host_name': self.spider.host_name,
                                           'task_success_count': task_success_count,
                                           'task_fail_count': task_fail_count,
                                           'insert_success_count': insert_success_count,
                                           'insert_repeat_count': insert_repeat_count,
                                           'insert_fail_count': insert_fail_count,
                                           'update_success_count': update_success_count,
                                           'update_fail_count': update_fail_count,
                                           'timestamp': timestamp},
                                     timeout=30)
            if resp.status_code != 200:
                logger.error(f'发送统计数据异常:{resp.status_code} {resp.text}')
        except Exception as e:
            logger.error(f'发送统计数据异常:{e}')
        logger.debug(f'{self.spider.spider_id} {self.spider.spider_name} 成功任务数:{task_success_count}'
                     f' 失败任务数:{task_fail_count} 入库数:{insert_success_count} 重复数:{insert_repeat_count} '
                     f'入库失败数:{insert_fail_count} 更新数:{update_success_count} 更新失败数:{update_fail_count}'
                     )

    def _monitor(self):
        """
        监控守护线程
        """
        self._heartbeat(0)
        while self.__monitor:
            self._heartbeat(1)
            self._report()
        self._heartbeat(-1)
        self._report()
        self.__monitor_complete = True

    async def start(self, monitor: bool):
        """
        开始任务调度
        """
        self.__monitor = monitor
        logger.info(f'开始调度...')
        if monitor:
            logger.info(f'注册爬虫...')
            monitor_thread = Thread(target=self._monitor)
            monitor_thread.daemon = True  # 设置为守护线程
            monitor_thread.start()

    async def stop(self):
        """
        结束任务调度
        """
        self.__monitor = False
        while not self.__monitor_complete:
            await asyncio.sleep(1)
        logger.info(f'结束调度...')




from .crawl import crawl
from .downloade.downloader import Downloader
from .downloade.aiohttp_downloader import Downloader as AioHttpDownloader
from .common import logger
from scan.core.spiders.rabbit_spider import RabbitSpider as Spider
from scan.core.spiders.simple_spider import SimpleSpider
from .config import Config
from .monitor import monitor




from scan.downloade.downloader import Downloader
from scan.downloade.aiohttp_downloader import Downloader as AioHttpDownloader


class Crawl:
    def __init__(self):
        self.downloader = Downloader()
        self.aiohttp_downloader = AioHttpDownloader()

    async def fetch(self,  url, params=None, data=None, files=None, json=None, content=None, headers=None, cookies=None,
                    verify=True, http2=False, auth=None, proxies=None, allow_redirects=True, stream=False,
                    session=False, timeout=30, cycle=1, tls=False, use_aiohttp=False):
        if use_aiohttp:
            res = await self.aiohttp_downloader.request(
                url, params=params, data=data, files=files, json=json, content=content, headers=headers,
                proxies=proxies,
                verify=verify, http2=http2, cookies=cookies, auth=auth, allow_redirects=allow_redirects,
                timeout=timeout,
                cycle=cycle, stream=stream, tls=tls, session=session
            )
        else:
            res = await self.downloader.request(
                url, params=params, data=data, files=files, json=json, content=content, headers=headers,
                proxies=proxies, verify=verify, http2=http2, cookies=cookies, auth=auth,
                allow_redirects=allow_redirects, timeout=timeout, cycle=cycle, stream=stream, tls=tls, session=session
            )
        return res

    async def close(self):
        await self.downloader.close_all()


crawl = Crawl()
__all__ = crawl



import os
import sys
import platform
from scan.log import get_logger


if platform.system().lower() == 'windows':
    sn = 'hscan'
else:
    try:
        file_path = os.path.abspath(sys.argv[0])
        sn = file_path.split('/')[-1].replace('.py', '')
    except:
        sn = 'hscan'

logger = get_logger(sn)
